<?php
/*****************************************************************************
 *  Copyright notice
 *
 *  ⓒ 2013 Michiel Roos <michiel@donationbasedhosting.org>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is free
 *  software; you can redistribute it and/or modify it under the terms of the
 *  GNU General Public License as published by the Free Software Foundation;
 *  either version 2 of the License, or (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 *  more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ****************************************************************************/

if (!defined('TYPO3_cliMode')) {
	die('Access denied: CLI only.');
}

require_once PATH_t3lib . 'class.t3lib_cli.php';

/**
 * General CLI dispatcher for the tableupdater extension.
 *
 * @package tableupdater
 * @author Michiel Roos <michiel@donationbasedhosting.org>
 */
class Tx_Tableupdater_Cli_Dispatcher extends t3lib_cli {

	/**
	 * Extension key
	 */
	const EXTKEY = 'tableupdater';

	/**
	 * Creates this object.
	 */
	public function __construct() {
		parent::__construct();
		$this->cli_help = array_merge($this->cli_help, array(
			'name' => 'tx_tableupdater',
			'synopsis' => self::EXTKEY . ' [update|delete|updateAndDelete]',
			'description' => 'Create missing tables, add missing fields, alter changed fields and cleanup deprecated fields and tables.',
			'examples' => 'typo3/cli_dispatch.phpsh tableupdater update',
			'author' => '(c) 2013 Donation Based Hosting <michiel@donationbasedhosting.org>',
		));
	}

	/**
	 * Update tables in database
	 *
	 * @throws Exception
	 * @return void
	 */
	public function main() {
		$action = (string)$this->cli_args['_DEFAULT'][1];

		if (!in_array($action, array('update', 'delete', 'updateAndDelete'))) {
			$this->cli_echo('Action "' . $action . '" is not supported.' . LF . LF);
			$this->cli_help();
			exit();
		}

		$tblFileContent = t3lib_div::getUrl(PATH_t3lib . 'stddb/tables.sql');

		foreach ($GLOBALS['TYPO3_LOADED_EXT'] as $loadedExtConf) {
			if (is_array($loadedExtConf) && $loadedExtConf['ext_tables.sql']) {
				$tblFileContent .= LF . LF . LF . LF . t3lib_div::getUrl($loadedExtConf['ext_tables.sql']);
			}
		}

		if (t3lib_utility_VersionNumber::convertVersionNumberToInteger(TYPO3_version) < 4007000) {
			$installer = t3lib_div::makeInstance('t3lib_install');
		} else {
			$installer = t3lib_div::makeInstance('t3lib_install_Sql');
			$tblFileContent .= t3lib_cache::getDatabaseTableDefinitions();
		}

		$fileContent = implode(
			LF,
			$installer->getStatementArray($tblFileContent, 1, '^CREATE TABLE ')
		);
		$definitionsFromFile = $installer->getFieldDefinitions_fileContent($fileContent);
		if (!count($definitionsFromFile)) {
			$this->cli_echo("Error: There were no 'CREATE TABLE' definitions in the provided file");
			exit();
		}

		$definitionsFromDatabase = $installer->getFieldDefinitions_database();
		$diff = $installer->getDatabaseExtra($definitionsFromFile, $definitionsFromDatabase);
		$updateStatements = $installer->getUpdateSuggestions($diff);

		$diff = $installer->getDatabaseExtra($definitionsFromDatabase, $definitionsFromFile);
		$removeStatements = $installer->getUpdateSuggestions($diff, 'remove');

		$results = array();
		if (($action === 'update' || $action === 'updateAndDelete') && count($updateStatements)) {
			$results[] = $this->performUpdateQueries($updateStatements['clear_table']);
			$results[] = $this->performUpdateQueries($updateStatements['clear_table']);
			$results[] = $this->performUpdateQueries($updateStatements['add']);
			$results[] = $this->performUpdateQueries($updateStatements['change']);
			$results[] = $this->performUpdateQueries($updateStatements['create_table']);
		}

		if (($action === 'delete' || $action === 'updateAndDelete') && count($removeStatements)) {
			$results[] = $this->performUpdateQueries($removeStatements['change']);
			$results[] = $this->performUpdateQueries($removeStatements['drop']);
			$results[] = $this->performUpdateQueries($removeStatements['change_table']);
			$results[] = $this->performUpdateQueries($removeStatements['drop_table']);
		}

		$return = '';
		foreach ($results as $result) {
			if (is_array($result)) {
				$return .= implode(LF, $result);
			}
		}
		$this->cli_echo($return);
	}

	/**
	 * Performs the queries passed from the input array.
	 *
	 * @param   array  Array of SQL queries to execute.
	 *
	 * @return  mixed  Array with error message from database if any occured.
	 *                   Otherwise true if everything was executed successfully.
	 */
	function performUpdateQueries($arr) {
		$result = array();
		if (is_array($arr)) {
			foreach ($arr as $key => $string) {
				$res = $GLOBALS['TYPO3_DB']->admin_query($string);
				if ($res === FALSE) {
					$result[$key] = $GLOBALS['TYPO3_DB']->sql_error();
				} elseif (is_resource($res)) {
					$GLOBALS['TYPO3_DB']->sql_free_result($res);
				}
			}
		}
		if (count($result) > 0) {
			return $result;
		} else {
			return TRUE;
		}
	}
}

echo t3lib_div::makeInstance('Tx_Tableupdater_Cli_Dispatcher')->main() . LF;

?>